# Optima Analyses------------------------

# 1 Preparation--------------------------

# dat = read.csv("Data_withIST.csv", dec = ".")
dat = read.csv("Data_noIST.csv", dec = ".")

library("lavaan")
library("psych")
library("ggplot2")
library("RColorBrewer")
library("EffectLiteR")


# 2 LCS function/model definitions-------

# Define (unconditional) Univariate Latent Change Score model 

ULCSu <- function(t1,t2,data) {
  
  d <- data[,c(t1,t2)]
  
  names(d) <- c("X_T1","X_T2")
  
  model<-'

X_T2 ~ 1*X_T1     # Fixed regression of X_T2 on X_T1
dX1 =~ 1*X_T2     # Fixed regression of dX1 on X_T2
X_T2 ~ 0*1        # This line constrains the intercept of X_T2 to 0
X_T2 ~~ 0*X_T2    # This fixes the variance of the X_T2 to 0 

dX1 ~ mean_change*1        # This estimates the intercept of the change scores 
X_T1 ~  1                  # This estimates the intercept of X_T1 
dX1 ~~  var_change*dX1     # This estimates the variance of the change scores 
X_T1 ~~   X_T1             # This estimates the variance of X_T1 
dX1~~X_T1                  # This estimates the covariance between change and baseline

standard_change := mean_change/sqrt(var_change)   # This estimates the standardized mean change
'
  
  fit <- lavaan(model, data=d, estimator='mlr',fixed.x=FALSE,missing='fiml')
  summary(fit, fit.measures=TRUE, standardized=TRUE, rsquare=TRUE, ci = TRUE)
}



# Define an (unconditional) multiple (three) indicator Univariate Latent Change Score model
# with equal slopes and intercepts across time (but unequal residual variances)

MIULCSu <- function(t1p1,t1p2,t1p3,t2p1,t2p2,t2p3,data) {
  
  d <- data[,c(t1p1,t1p2,t1p3,t2p1,t2p2,t2p3)]
  
  names(d) <- c("T1X1","T1X2","T1X3","T2X1","T2X2","T2X3")
  
  model<-'

  X_T1=~1*T1X1+T1X2+T1X3                           # This specifies the measurement model for X_T1 
  X_T2=~1*T2X1+equal("X_T1=~T1X2")*T2X2+equal("X_T1=~T1X3")*T2X3   # This specifies the measurement model for X_T2 with the equality constrained factor loadings
  
  X_T2 ~ 1*X_T1     # Fixed regression of X_T2 on X_T1
  dX1 =~ 1*X_T2     # Fixed regression of dX1 on X_T2
  X_T2 ~ 0*1        # This line constrains the intercept of X_T2 to 0
  X_T2 ~~ 0*X_T2    # This fixes the variance of the X_T2 to 0 
  
  dX1 ~ mean_change*1      # This estimates the intercept of the change score 
  X_T1 ~  1                # This estimates the intercept of X_T1 
  dX1 ~~  var_change*dX1   # This estimates the variance of the change scores 
  X_T1 ~~   X_T1           # This estimates the variance of the X_T1 
  dX1~~X_T1                # This estimates the covariance between change and baseline
  
  standard_change := mean_change/sqrt(var_change)
  
  T1X1~~T2X1   # This allows residual covariance on indicator X1 across T1 and T2
  T1X2~~T2X2   # This allows residual covariance on indicator X2 across T1 and T2
  T1X3~~T2X3   # This allows residual covariance on indicator X3 across T1 and T2
  
  T1X1~~T1X1   # This allows residual variance on indicator X1 
  T1X2~~T1X2   # This allows residual variance on indicator X2
  T1X3~~T1X3   # This allows residual variance on indicator X3
  
  T2X1~~T2X1  # This allows residual variance on indicator X1 at T2 
  T2X2~~T2X2  # This allows residual variance on indicator X2 at T2 
  T2X3~~T2X3  # This allows residual variance on indicator X3 at T2
  
  T1X1~0*1                 # This constrains the intercept of X1 to 0 at T1
  T1X2~1                   # This estimates the intercept of X2 at T1
  T1X3~1                   # This estimates the intercept of X3 at T1
  T2X1~0*1                 # This constrains the intercept of X1 to 0 at T2
  T2X2~equal("T1X2~1")*1   # This estimates the intercept of X2 at T2
  T2X3~equal("T1X3~1")*1   # This estimates the intercept of X3 at T2
  
  '
  
  fit <- lavaan(model, data=d, estimator='mlr',fixed.x=FALSE,missing='fiml')
  summary(fit, fit.measures=TRUE, standardized=TRUE, rsquare=TRUE, ci = TRUE)
  
}


# Define a multiple (three) indicator Univariate Latent Change Score model including effects of a covariate and self-feedback
# with equal slopes and intercepts across time (but unequal residual variances)

MIULCSc2 <- function(t1p1,t1p2,t1p3,t2p1,t2p2,t2p3,cov,data) {
  
  d <- data[,c(t1p1,t1p2,t1p3,t2p1,t2p2,t2p3,cov)]
  
  names(d) <- c("T1X1","T1X2","T1X3","T2X1","T2X2","T2X3","z")
  
  model<-'

  X_T1=~1*T1X1+T1X2+T1X3                           # This specifies the measurement model for X_T1 
  X_T2=~1*T2X1+equal("X_T1=~T1X2")*T2X2+equal("X_T1=~T1X3")*T2X3   # This specifies the measurement model for X_T2 with the equality constrained factor loadings
  
  X_T2 ~ 1*X_T1     # Fixed regression of X_T2 on X_T1
  dX1 =~ 1*X_T2     # Fixed regression of dX1 on X_T2
  X_T2 ~ 0*1        # This line constrains the intercept of X_T2 to 0
  X_T2 ~~ 0*X_T2    # This fixes the variance of the X_T2 to 0 
  
  dX1 ~ 1           # This estimates the intercept of the change score 
  X_T1 ~  1         # This estimates the intercept of X_T1 
  dX1 ~~  dX1       # This estimates the variance of the change scores 
  X_T1 ~~   X_T1    # This estimates the variance of the X_T1 
  dX1~ X_T1         # This estimates the self-feedback parameter
  
  z ~~ z            # Variance
  z ~ 1             # Intercept
  X_T1 ~ z          # Effect on Baseline Score
  dX1 ~ z           # Effect on Change Score
  
  T1X1~~T2X1   # This allows residual covariance on indicator X1 across T1 and T2
  T1X2~~T2X2   # This allows residual covariance on indicator X2 across T1 and T2
  T1X3~~T2X3   # This allows residual covariance on indicator X3 across T1 and T2
  
  T1X1~~T1X1   # This allows residual variance on indicator X1 
  T1X2~~T1X2   # This allows residual variance on indicator X2
  T1X3~~T1X3   # This allows residual variance on indicator X3
  
  T2X1~~T2X1  # This allows residual variance on indicator X1 at T2 
  T2X2~~T2X2  # This allows residual variance on indicator X2 at T2 
  T2X3~~T2X3  # This allows residual variance on indicator X3 at T2
  
  T1X1~0*1                 # This constrains the intercept of X1 to 0 at T1
  T1X2~1                   # This estimates the intercept of X2 at T1
  T1X3~1                   # This estimates the intercept of X3 at T1
  T2X1~0*1                 # This constrains the intercept of X1 to 0 at T2
  T2X2~equal("T1X2~1")*1   # This estimates the intercept of X2 at T2
  T2X3~equal("T1X3~1")*1   # This estimates the intercept of X3 at T2
  
  '
  
  fit <- lavaan(model, data=d, estimator='mlr',fixed.x=FALSE,missing='fiml')
  summary(fit, fit.measures=TRUE, standardized=TRUE, rsquare=TRUE, ci = TRUE)
  
}


# Fit the multiple (three) indicator bivariate Latent Change Score model 
# with equal slopes and intercepts across time (but unequal residual variances)

MIBLCS <- function(xt1p1,xt1p2,xt1p3,xt2p1,xt2p2,xt2p3,
                   yt1p1,yt1p2,yt1p3,yt2p1,yt2p2,yt2p3,
                   data) {
  
  d <- data[,c(xt1p1,xt1p2,xt1p3,xt2p1,xt2p2,xt2p3,
               yt1p1,yt1p2,yt1p3,yt2p1,yt2p2,yt2p3)]
  
  names(d) <- c("T1X1","T1X2","T1X3","T2X1","T2X2","T2X3",
                "T1Y1","T1Y2","T1Y3","T2Y1","T2Y2","T2Y3")
  
  model<-'

X_T1=~1*T1X1+T1X2+T1X3                           # This specifies the measurement model for X_T1 
X_T2=~1*T2X1+equal("X_T1=~T1X2")*T2X2+equal("X_T1=~T1X3")*T2X3   # This specifies the measurement model for X_T2 with the equality constrained factor loadings

X_T2 ~ 1*X_T1     # Fixed regression of X_T2 on X_T1
dX1 =~ 1*X_T2     # Fixed regression of dX1 on X_T2
X_T2 ~ 0*1        # This line constrains the intercept of X_T2 to 0
X_T2 ~~ 0*X_T2    # This fixes the variance of the X_T2 to 0 

dX1 ~ 1           # This estimates the intercept of the change score 
X_T1 ~  1         # This estimates the intercept of X_T1 
dX1 ~~  dX1       # This estimates the variance of the change scores 
X_T1 ~~   X_T1    # This estimates the variance of the X_T1 

T1X1~~T2X1   # This allows residual covariance on indicator X1 across T1 and T2
T1X2~~T2X2   # This allows residual covariance on indicator X2 across T1 and T2
T1X3~~T2X3   # This allows residual covariance on indicator X3 across T1 and T2

T1X1~~T1X1   # This allows residual variance on indicator X1 
T1X2~~T1X2   # This allows residual variance on indicator X2
T1X3~~T1X3   # This allows residual variance on indicator X3

T2X1~~T2X1  # This allows residual variance on indicator X1 at T2 
T2X2~~T2X2  # This allows residual variance on indicator X2 at T2 
T2X3~~T2X3  # This allows residual variance on indicator X3 at T2

T1X1~0*1                 # This constrains the intercept of X1 to 0 at T1
T1X2~1                   # This estimates the intercept of X2 at T1
T1X3~1                   # This estimates the intercept of X3 at T1
T2X1~0*1                 # This constrains the intercept of X1 to 0 at T2
T2X2~equal("T1X2~1")*1   # This estimates the intercept of X2 at T2
T2X3~equal("T1X3~1")*1   # This estimates the intercept of X3 at T2

Y_T1=~1*T1Y1+T1Y2+T1Y3                           # This specifies the measurement model for Y_T1 
Y_T2=~1*T2Y1+equal("Y_T1=~T1Y2")*T2Y2+equal("Y_T1=~T1Y3")*T2Y3   # This specifies the measurement model for Y_T2 with the equality constrained factor loadings

Y_T2 ~ 1*Y_T1     # Fixed regression of Y_T2 on Y_T1
dY1 =~ 1*Y_T2     # Fixed regression of dY1 on Y_T2
Y_T2 ~ 0*1        # This line constrains the intercept of Y_T2 to 0
Y_T2 ~~ 0*Y_T2    # This fixes the variance of the Y_T2 to 0 

dY1 ~ 1           # This estimates the intercept of the change score 
Y_T1 ~  1         # This estimates the intercept of Y_T1 
dY1 ~~  dY1       # This estimates the variance of the change scores 
Y_T1 ~~   Y_T1    # This estimates the variance of the Y_T1 

T1Y1~~T2Y1   # This allows residual covariance on indicator Y1 across T1 and T2
T1Y2~~T2Y2   # This allows residual covariance on indicator Y2 across T1 and T2
T1Y3~~T2Y3   # This allows residual covariance on indicator Y3 across T1 and T2

T1Y1~~T1Y1   # This allows residual variance on indicator Y1 
T1Y2~~T1Y2   # This allows residual variance on indicator Y2
T1Y3~~T1Y3   # This allows residual variance on indicator Y3

T2Y1~~T2Y1  # This allows residual variance on indicator Y1 at T2 
T2Y2~~T2Y2  # This allows residual variance on indicator Y2 at T2 
T2Y3~~T2Y3  # This allows residual variance on indicator Y3 at T2

T1Y1~0*1                 # This constrains the intercept of Y1 to 0 at T1
T1Y2~1                   # This estimates the intercept of Y2 at T1
T1Y3~1                   # This estimates the intercept of Y3 at T1
T2Y1~0*1                 # This constrains the intercept of Y1 to 0 at T2
T2Y2~equal("T1Y2~1")*1   # This estimates the intercept of Y2 at T2
T2Y3~equal("T1Y3~1")*1   # This estimates the intercept of Y3 at T2

dX1~X_T1+Y_T1   # This estimates the Y to X coupling parameter and the X to X self-feedback
dY1~Y_T1+X_T1   # This estimates the X to Y coupling parameter and the Y to Y self-feedback

X_T1 ~~ Y_T1    # This estimates the baseline covariance
dX1 ~~ dY1      # This estimates the (residual) change score covariance

'
  fit <- lavaan(model, data=d, estimator='mlr',fixed.x=FALSE,missing='fiml')
  summary(fit, fit.measures=TRUE, standardized=TRUE, rsquare=TRUE, ci = TRUE)
  
}


# Fit the (unconditional) multiple (three) indicator bivariate Latent Change Score model 
# with equal slopes and intercepts across time (but unequal residual variances)

MIBLCSu <- function(xt1p1,xt1p2,xt1p3,xt2p1,xt2p2,xt2p3,
                    yt1p1,yt1p2,yt1p3,yt2p1,yt2p2,yt2p3,
                    data) {
  
  d <- data[,c(xt1p1,xt1p2,xt1p3,xt2p1,xt2p2,xt2p3,
               yt1p1,yt1p2,yt1p3,yt2p1,yt2p2,yt2p3)]
  
  names(d) <- c("T1X1","T1X2","T1X3","T2X1","T2X2","T2X3",
                "T1Y1","T1Y2","T1Y3","T2Y1","T2Y2","T2Y3")
  
  model<-'

X_T1=~1*T1X1+T1X2+T1X3                           # This specifies the measurement model for X_T1 
X_T2=~1*T2X1+equal("X_T1=~T1X2")*T2X2+equal("X_T1=~T1X3")*T2X3   # This specifies the measurement model for X_T2 with the equality constrained factor loadings

X_T2 ~ 1*X_T1     # Fixed regression of X_T2 on X_T1
dX1 =~ 1*X_T2     # Fixed regression of dX1 on X_T2
X_T2 ~ 0*1        # This line constrains the intercept of X_T2 to 0
X_T2 ~~ 0*X_T2    # This fixes the variance of the X_T2 to 0 

dX1 ~ 1           # This estimates the intercept of the change score 
X_T1 ~  1         # This estimates the intercept of X_T1 
dX1 ~~  dX1       # This estimates the variance of the change scores 
X_T1 ~~   X_T1    # This estimates the variance of the X_T1 

T1X1~~T2X1   # This allows residual covariance on indicator X1 across T1 and T2
T1X2~~T2X2   # This allows residual covariance on indicator X2 across T1 and T2
T1X3~~T2X3   # This allows residual covariance on indicator X3 across T1 and T2

T1X1~~T1X1   # This allows residual variance on indicator X1 
T1X2~~T1X2   # This allows residual variance on indicator X2
T1X3~~T1X3   # This allows residual variance on indicator X3

T2X1~~T2X1  # This allows residual variance on indicator X1 at T2 
T2X2~~T2X2  # This allows residual variance on indicator X2 at T2 
T2X3~~T2X3  # This allows residual variance on indicator X3 at T2

T1X1~0*1                 # This constrains the intercept of X1 to 0 at T1
T1X2~1                   # This estimates the intercept of X2 at T1
T1X3~1                   # This estimates the intercept of X3 at T1
T2X1~0*1                 # This constrains the intercept of X1 to 0 at T2
T2X2~equal("T1X2~1")*1   # This estimates the intercept of X2 at T2
T2X3~equal("T1X3~1")*1   # This estimates the intercept of X3 at T2

Y_T1=~1*T1Y1+T1Y2+T1Y3                           # This specifies the measurement model for Y_T1 
Y_T2=~1*T2Y1+equal("Y_T1=~T1Y2")*T2Y2+equal("Y_T1=~T1Y3")*T2Y3   # This specifies the measurement model for Y_T2 with the equality constrained factor loadings

Y_T2 ~ 1*Y_T1     # Fixed regression of Y_T2 on Y_T1
dY1 =~ 1*Y_T2     # Fixed regression of dY1 on Y_T2
Y_T2 ~ 0*1        # This line constrains the intercept of Y_T2 to 0
Y_T2 ~~ 0*Y_T2    # This fixes the variance of the Y_T2 to 0 

dY1 ~ 1           # This estimates the intercept of the change score 
Y_T1 ~  1         # This estimates the intercept of Y_T1 
dY1 ~~  dY1       # This estimates the variance of the change scores 
Y_T1 ~~   Y_T1    # This estimates the variance of the Y_T1 

T1Y1~~T2Y1   # This allows residual covariance on indicator Y1 across T1 and T2
T1Y2~~T2Y2   # This allows residual covariance on indicator Y2 across T1 and T2
T1Y3~~T2Y3   # This allows residual covariance on indicator Y3 across T1 and T2

T1Y1~~T1Y1   # This allows residual variance on indicator Y1 
T1Y2~~T1Y2   # This allows residual variance on indicator Y2
T1Y3~~T1Y3   # This allows residual variance on indicator Y3

T2Y1~~T2Y1  # This allows residual variance on indicator Y1 at T2 
T2Y2~~T2Y2  # This allows residual variance on indicator Y2 at T2 
T2Y3~~T2Y3  # This allows residual variance on indicator Y3 at T2

T1Y1~0*1                 # This constrains the intercept of Y1 to 0 at T1
T1Y2~1                   # This estimates the intercept of Y2 at T1
T1Y3~1                   # This estimates the intercept of Y3 at T1
T2Y1~0*1                 # This constrains the intercept of Y1 to 0 at T2
T2Y2~equal("T1Y2~1")*1   # This estimates the intercept of Y2 at T2
T2Y3~equal("T1Y3~1")*1   # This estimates the intercept of Y3 at T2

dX1~~X_T1         # This estimates the covariance between change in X and baseline X
dY1~~Y_T1         # This estimates the covariance between change in Y baseline Y
dX1~~Y_T1         # This estimates the covariance between change in X and baseline Y
dY1~~X_T1         # This estimates the covariance between change in Y and baseline X

X_T1 ~~ Y_T1    # This estimates the baseline covariance
dX1 ~~ dY1      # This estimates the (residual) change score covariance

'
  fit <- lavaan(model, data=d, estimator='mlr',fixed.x=FALSE,missing='fiml')
  summary(fit, fit.measures=TRUE, standardized=TRUE, rsquare=TRUE, ci = TRUE)
  
}



# 3 Data Preparation---------------------

# compute BDI parcels

dat$t0_bdi_p1 <- rowMeans(dat[,paste0("t0_bdi_",seq(1,21,3),".x")], na.rm = TRUE)
dat$t0_bdi_p2 <- rowMeans(dat[,paste0("t0_bdi_",seq(2,21,3),".x")], na.rm = TRUE)
dat$t0_bdi_p3 <- rowMeans(dat[,paste0("t0_bdi_",seq(3,21,3),".x")], na.rm = TRUE)

dat$t7_bdi_p1 <- rowMeans(dat[,paste0("t7_bdi_",seq(1,21,3),".x")], na.rm = TRUE)
dat$t7_bdi_p2 <- rowMeans(dat[,paste0("t7_bdi_",seq(2,21,3),".x")], na.rm = TRUE)
dat$t7_bdi_p3 <- rowMeans(dat[,paste0("t7_bdi_",seq(3,21,3),".x")], na.rm = TRUE)


# Get internal consistencies of parcels
psych::alpha(dat[,paste0("t0_bdi_",seq(1,21,3),".x")])$total
psych::alpha(dat[,paste0("t0_bdi_",seq(2,21,3),".x")])$total
psych::alpha(dat[,paste0("t0_bdi_",seq(3,21,3),".x")])$total

psych::alpha(dat[,paste0("t7_bdi_",seq(1,21,3),".x")])$total
psych::alpha(dat[,paste0("t7_bdi_",seq(2,21,3),".x")])$total
psych::alpha(dat[,paste0("t7_bdi_",seq(3,21,3),".x")])$total


# Get pre- to post-treatment correlations of parcels
with(dat, cor(t0_bdi_p1, t7_bdi_p1, method = "pearson", use = "complete.obs"))
with(dat, cor(t0_bdi_p2, t7_bdi_p2, method = "pearson", use = "complete.obs"))
with(dat, cor(t0_bdi_p3, t7_bdi_p3, method = "pearson", use = "complete.obs"))


# Get pre- to post-treatment correlations of PID domains
with(dat, cor(t0_neg_aff, t7_neg_aff, method = "pearson", use = "complete.obs"))
with(dat, cor(t0_detach, t7_detach, method = "pearson", use = "complete.obs"))
with(dat, cor(t0_antago, t7_antago, method = "pearson", use = "complete.obs"))
with(dat, cor(t0_disinh, t7_disinh, method = "pearson", use = "complete.obs"))
with(dat, cor(t0_psycho, t7_psycho, method = "pearson", use = "complete.obs"))


# 4 LCS analyses-------------------------

# 4.1 (unconditional) Univariate LCS-----

# define involved variables 

t0 <- c("t0_anx", "t0_emolab","t0_host","t0_pers","t0_resaff","t0_sepins","t0_sub",
        "t0_anhe", "t0_depr","t0_intim", "t0_susp","t0_with",
        "t0_attsee","t0_call","t0_deceit","t0_grand","t0_manip",
        "t0_distr","t0_impu","t0_irres","t0_riperf","t0_risk",  
        "t0_ecce", "t0_perc", "t0_unus")
t7 <- c("t7_anx", "t7_emolab","t7_host","t7_pers","t7_resaff","t7_sepins","t7_sub",
        "t7_anhe", "t7_depr","t7_intim", "t7_susp","t7_with",
        "t7_attsee","t7_call","t7_deceit","t7_grand","t7_manip",
        "t7_distr","t7_impu","t7_irres","t7_riperf","t7_risk",  
        "t7_ecce", "t7_perc", "t7_unus")

# prepare results object

results <- data.frame(matrix(NA,25,10))
colnames(results) <- c("MC.est","MC.se","MC.pvalue","MC.lower","MC.upper",
                       "SMC.est","SMC.lower","SMC.upper",
                       "r","r.pvalue")
rownames(results) <- substr(t0,4,10)

# conduct analyses and save results

for (i in 1:25) {
  fit <- ULCSu(t0[i],t7[i],dat)
  results[i,1:5] <- fit$PE[5,c("est","se","pvalue","ci.lower","ci.upper")]  # save mean change
  results[i,6:8] <- fit$PE[10,c("est","ci.lower","ci.upper")]  # save standardized mean change
  results[i,9:10] <- fit$PE[9,c("std.all","pvalue")]  # save correlation between baseline and change
}

# show and output results

round(results,digits = 3)
write.csv2(round(results,digits = 3),file = "facets_ULCSu.csv")


# visualize results

col <- brewer.pal(n = 5,name = "Set1") #

results$name <- c("Anxiousness","Emotional Lability", "Hostility","Perseveration","Restricted Affectivity","Separation Insecurity","Submissiveness",
                  "Anhedonia", "Depressivity","Intimacy Avoidance","Suspiciousness","Withdrawal",
                  "Attention Seeking","Callousness","Deceitfulness","Grandiosity","Manipulativeness", 
                  "Distractibility","Impulsivity","Irresponsibility", "Rigid Perfectionism","Risk Taking",
                  "Eccentricity","Perceptual Dysregulation","Unusual Experiences")
results$domain <- c("NA", "NA", "NA", "NA", "NA", "NA", "NA",
                    "DET", "DET", "DET", "DET", "DET",
                    "ANT","ANT","ANT","ANT","ANT",
                    "DIS","DIS","DIS","DIS","DIS",
                    "PSY","PSY","PSY")
results$rank <- order(results$SMC.est)
results <- results[results$rank,]
results$nr <- factor(1:25)

plot <- ggplot(results,aes(x = rev(nr), y= SMC.est,fill = domain)) +
  geom_bar(stat = "identity", color = "black",width = 0.8) + 
  geom_errorbar(aes(ymin = SMC.lower, ymax = SMC.upper),color ="black", width = 0.2) +
  scale_y_continuous(breaks = c(-0.8,-0.5,-0.2,0,0.2)) +
  scale_x_discrete(labels = rev(results$name)) +
  scale_fill_manual(values = col, name = "") +
  xlab("Facets") + ylab("Standardized change") + 
  coord_flip(ylim = c(-0.9,0.3)) + theme_minimal() + theme(legend.position="right") 

png(file="facet_fig.png", width=15, height=10, units = "cm", res = 800)
plot
dev.off()


# 4.2 Multiple indicator ULCS------------

# Fit an (unconditional) multiple (three) indicator Univariate Latent Change Score model for Negative Affectivity 
#
# MIULCSu(t1p1,t1p2,t1p3,t2p1,t2p2,t2p3,data)


# define involved variables 

domains = c("neg_aff", "detach", "antago", "disinh", "psycho")
facets = c("anx","emolab","sepins", # neg_aff
           "anhe", "intim", "with", # detach
           "manip", "deceit", "grand", # antago
           "irres", "impu", "distr", # disinh
           "ecce", "perc", "unus") # psycho

t0 <- paste0("t0_", facets) 
t7 <- paste0("t7_", facets)


# prepare results object

results <- data.frame(matrix(NA,5,22))
colnames(results) <- c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled",
                       "rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr",
                       paste0("t0_", 1:3, ".std"), paste0("t7_", 1:3, ".std"),
                       paste0("SMC.", c("est","se","pvalue","lower","upper")),
                       paste0("change_cor.", c("std","pvalue")))
rownames(results) <- domains

# conduct analyses and save results

for (row in 1:5) {
  i = row * 3 - 2
  j = i + 1
  k = i + 2
  fit <- MIULCSu(t0[i],t0[j],t0[k],
               t7[i],t7[j],t7[k],
               dat)
  results[row,1:9] <- fit$FIT[c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled","rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr")]  # model fit
  results[row,10:15] <- fit$PE[1:6,c("std.all")] # standardized loadings of indicators
  results[row,16:20] <- fit$PE[31,c("est","se","pvalue","ci.lower","ci.upper")]  # standardized mean change
  results[row,21:22] <- fit$PE[15,c("std.all","pvalue")]  # correlation between baseline and change
}

# show and output results

round(results,digits = 3)
write.csv2(round(results,digits = 3),file = "MIULCSu.csv")


# visualize results

col <- brewer.pal(n = 5,name = "Set1") #

results$name <- c("Negative Affectivity", "Detachment", "Antagonism", "Disinhibition", "Psychoticism")

results$rank <- order(results$SMC.est)
results <- results[results$rank,]
results$nr <- factor(1:5)

plot <- ggplot(results,aes(x = rev(nr), y= SMC.est,fill = name)) +
  geom_bar(stat = "identity", color = "black",width = 0.8) + 
  geom_errorbar(aes(ymin = SMC.lower, ymax = SMC.upper),color ="black", width = 0.2) +
  scale_y_continuous(breaks = c(-0.8,-0.5,-0.2,0,0.2)) +
  scale_x_discrete(labels = rev(results$name)) +
  scale_fill_manual(values = col, name = "") +
  xlab("Domains") + ylab("Standardized change") + 
  coord_flip(ylim = c(-1.2,0.3)) + theme_minimal() + theme(legend.position="none") 

png(file="domain_fig.png", width=15, height=10, units = "cm", res = 800)
plot
dev.off()




# 4.3 MI-ULCS+Treatment------------------

# Define a multiple (three) indicator Univariate Latent Change Score model including effects of a covariate and self-feedback
# with equal slopes and intercepts across time (but unequal residual variances)

# MIULCSc2(t1p1,t1p2,t1p3,t2p1,t2p2,t2p3,cov,data)


# define involved variables 

domains = c("neg_aff", "detach", "antago", "disinh", "psycho")
facets = c("anx","emolab","sepins", # neg_aff
           "anhe", "intim", "with", # detach
           "manip", "deceit", "grand", # antago
           "irres", "impu", "distr", # disinh
           "ecce", "perc", "unus") # psycho

t0 <- paste0("t0_", facets) 
t7 <- paste0("t7_", facets)


# prepare results object

results <- data.frame(matrix(NA,5,32))
colnames(results) <- c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled",
                       "rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr",
                       paste0("t0_", 1:3, ".std"), paste0("t7_", 1:3, ".std"),
                       paste0("SMC.", c("est","se","pvalue","lower","upper")),
                       "cov_t0.est","cov_t0.se","cov_t0.pvalue","cov_t0.lower","cov_t0.upper", "cov_t0_SMC.est",
                       "cov_d.est","cov_d.se","cov_d.pvalue","cov_d.lower","cov_d.upper", "cov_d_SMC.est")
rownames(results) <- domains

# conduct analyses and save results

for (row in 1:5) {
  i = row * 3 - 2
  j = i + 1
  k = i + 2
  fit <- MIULCSc2(t0[i],t0[j],t0[k],
                  t7[i],t7[j],t7[k],
                  "treatment",dat)
  results[row,1:9] <- fit$FIT[c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled","rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr")]  # model fit
  results[row,10:15] <- fit$PE[1:6,c("std.all")] # standardized loadings of indicators
  results[row,16:20] <- fit$PE[15,c("est","se","pvalue","ci.lower","ci.upper")]  # standardized mean change
  
  results[row,21:26] <- fit$PE[18,c("est","se","pvalue","ci.lower","ci.upper","std.all")] # effect of covariate on baseline score
  results[row,27:32] <- fit$PE[19,c("est","se","pvalue","ci.lower","ci.upper","std.all")] # effect of covariate on change
  
}

# show and output results

round(results,digits = 3)
write.csv2(round(results,digits = 3),file = "MIULCSc2.csv")




# 4.4 Multiple indicator BLCS------------

# Fit a multiple (three) indicator bivariate Latent Change Score model for Depressive Symptoms (X) and Negative Affectivity (Y)
#
# MIBLCS(xt1p1,xt1p2,xt1p3,xt2p1,xt2p2,xt2p3,yt1p1,yt1p2,yt1p3,yt2p1,yt2p2,yt2p3,data)



# define involved variables 

domains = c("neg_aff", "detach", "antago", "disinh", "psycho")
facets = c("anx","emolab","sepins", # neg_aff
           "anhe", "intim", "with", # detach
           "manip", "deceit", "grand", # antago
           "irres", "impu", "distr", # disinh
           "ecce", "perc", "unus") # psycho

t0 <- paste0("t0_", facets) 
t7 <- paste0("t7_", facets)


# prepare results object

results <- data.frame(matrix(NA,5,37))
colnames(results) <- c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled",
                       "rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr",
                       paste0("t0_bdi_", 1:3, ".std"), paste0("t7_bdi_", 1:3, ".std"),
                       paste0("t0_pid_", 1:3, ".std"), paste0("t7_pid_", 1:3, ".std"),
                       paste0("PID0_dBDI.", c("est","se","pvalue","lower","upper", "std")),
                       paste0("BDI0_dPID.", c("est","se","pvalue","lower","upper", "std")),
                       paste0("t0_cor.", c("std","pvalue")),
                       paste0("d_cor.", c("std","pvalue")))
rownames(results) <- domains

# conduct analyses and save results

for (row in 1:5) {
  i = row * 3 - 2
  j = i + 1
  k = i + 2
  fit <- MIBLCS("t0_bdi_p1","t0_bdi_p2","t0_bdi_p3",
                 "t7_bdi_p1","t7_bdi_p2","t7_bdi_p3",
                 t0[i],t0[j],t0[k],
                 t7[i],t7[j],t7[k],
                 dat)
  results[row,1:9] <- fit$FIT[c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled","rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr")]  # model fit
  results[row,10:21] <- fit$PE[c(1:6,30:35),c("std.all")] # standardized loadings of indicators
  results[row,22:27] <- fit$PE[60,c("est","se","pvalue","ci.lower","ci.upper","std.all")]  # (incremental) effect of baseline Y on change in X
  results[row,28:33] <- fit$PE[62,c("est","se","pvalue","ci.lower","ci.upper","std.all")]  # (incremental) effect of baseline X on change in Y
  results[row,34:35] <- fit$PE[63,c("std.all","pvalue")]  # correlation between X and Y at baseline
  results[row,36:37] <- fit$PE[64,c("std.all","pvalue")]  # correlation between (residualized) change in X and Y
}

# show and output results

round(results,digits = 3)
write.csv2(round(results,digits = 3),file = "MIBLCS.csv")



# 4.5 unconditional MIBLCS---------------

# Fit an unconditional multiple (three) indicator bivariate Latent Change Score model for Depressive Symptoms (X) and Negative Affectivity (Y)
#
# MIBLCSu(xt1p1,xt1p2,xt1p3,xt2p1,xt2p2,xt2p3,yt1p1,yt1p2,yt1p3,yt2p1,yt2p2,yt2p3,data)



# define involved variables 

domains = c("neg_aff", "detach", "antago", "disinh", "psycho")
facets = c("anx","emolab","sepins", # neg_aff
           "anhe", "intim", "with", # detach
           "manip", "deceit", "grand", # antago
           "irres", "impu", "distr", # disinh
           "ecce", "perc", "unus") # psycho

t0 <- paste0("t0_", facets) 
t7 <- paste0("t7_", facets)


# prepare results object

results <- data.frame(matrix(NA,5,27))
colnames(results) <- c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled",
                       "rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr",
                       paste0("t0_bdi_", 1:3, ".std"), paste0("t7_bdi_", 1:3, ".std"),
                       paste0("t0_pid_", 1:3, ".std"), paste0("t7_pid_", 1:3, ".std"),
                       paste0("cor_dBDI_PID0.", c("std","pvalue")),
                       paste0("cor_dPID_BDI0.", c("std","pvalue")),
                       paste0("cor_dBDI_dPID.", c("std","pvalue")))
rownames(results) <- domains

# conduct analyses and save results

for (row in 1:5) {
  i = row * 3 - 2
  j = i + 1
  k = i + 2
  fit <- MIBLCSu("t0_bdi_p1","t0_bdi_p2","t0_bdi_p3",
                 "t7_bdi_p1","t7_bdi_p2","t7_bdi_p3",
                 t0[i],t0[j],t0[k],
                 t7[i],t7[j],t7[k],
                 dat)
  results[row,1:9] <- fit$FIT[c("npar","chisq.scaled","df.scaled","pvalue.scaled","cfi.scaled","rmsea.scaled","rmsea.ci.lower.scaled","rmsea.ci.upper.scaled","srmr")]  # model fit
  results[row,10:21] <- fit$PE[c(1:6,30:35),c("std.all")] # standardized loadings of indicators
  results[row,22:23] <- fit$PE[61,c("std.all","pvalue")]  # correlation between change in X and Y at baseline
  results[row,24:25] <- fit$PE[62,c("std.all","pvalue")]  # correlation between change in Y and X at baseline
  results[row,26:27] <- fit$PE[64,c("std.all","pvalue")]  # correlation between change in X and change in Y
}

# show and output results

round(results,digits = 3)
write.csv2(round(results,digits = 3),file = "MIBLCSu.csv")



# 5 Effect Lite R analyses---------------

# only observed variables

fit <- effectLite(y = "t7_bdi", x = "treatment", z = 
                    c("t0_bdi", "t0_neg_aff", "t0_detach", "t0_antago" , "t0_disinh" , "t0_psycho"),
                  data = dat, method = "sem",  control = "KVT",missing = "FIML", estimator = "mlr",
                  interactions = "all")
fit
conditionalEffectsPlot(fit)
summary(fit@results@lavresults, fit.measures = TRUE)



# with latent outcome

measurement <- '
YT7 =~ 1*t7_bdi_p1+c(l_bdi_p2,l_bdi_p2)*t7_bdi_p2+c(l_bdi_p3,l_bdi_p3)*t7_bdi_p3      
YT0 =~ 1*t0_bdi_p1+c(l_bdi_p2,l_bdi_p2)*t0_bdi_p2+c(l_bdi_p3,l_bdi_p3)*t0_bdi_p3

t0_bdi_p1~~c(r_bdi_p1,r_bdi_p1)*t7_bdi_p1 
t0_bdi_p2~~c(r_bdi_p2,r_bdi_p2)*t7_bdi_p2 
t0_bdi_p3~~c(r_bdi_p3,r_bdi_p3)*t7_bdi_p3 

t0_bdi_p1~c(0,0)*1
t0_bdi_p2~c(i_bdi_p2,i_bdi_p2)*1
t0_bdi_p3~c(i_bdi_p3,i_bdi_p3)*1

t7_bdi_p1~c(0,0)*1
t7_bdi_p2~c(i_bdi_p2,i_bdi_p2)*1
t7_bdi_p3~c(i_bdi_p3,i_bdi_p3)*1
'

fit <- effectLite(y = "YT7", x = "treatment", z = 
                    c("YT0", "t0_neg_aff", "t0_detach", "t0_antago" , "t0_disinh" , "t0_psycho"),
                  measurement = measurement,data = dat, method = "sem",  control = "KVT",
                  homoscedasticity = TRUE,missing = "FIML", estimator = "mlr",
                  interactions = "all")

fit
conditionalEffectsPlot(fit)
summary(fit@results@lavresults, fit.measures = TRUE)



# 6 Descriptive statistics---------------
  
describe(dat[,c("t0_bdi", "t0_neg_aff", "t0_detach", "t0_antago" , "t0_disinh" , "t0_psycho",
                "t7_bdi", "t7_neg_aff", "t7_detach", "t7_antago" , "t7_disinh",  "t7_psycho")])



